/* Time Crack v1.2 -- Generic shared library for defeating
		      time-locked demonstration mode programs

  Released into the public domain

  by phundie@hackunix.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

time_t time(time_t *t);
unsigned int alarm(unsigned int seconds);
int getitimer(int which, struct itimerval *value);
int setitimer(int which, const  struct  itimerval  *value, \
    struct itimerval *ovalue);

int DIFF;
int GET_OPTS=1;
int NUM_CALLS=0;
int TIME_CRACK_DEBUG=0;
int TIME_CRACK_MAXRUNS=0;
time_t TIME_CRACK_MINEPOCH=0;
time_t TIME_CRACK_MAXEPOCH=0;
time_t START_TIME=0;

#ifdef XIMEOFDAY
int getximeofday(struct timeval *tv, struct timezone *tz)
#else
int gettimeofday(struct timeval *tv, struct timezone *tz)
#endif
{
	time_t h,c;
	int b,d,e;

	if (GET_OPTS)
	{
		GET_OPTS=0;
		if (getenv("TIME_CRACK_DEBUG"))
			TIME_CRACK_DEBUG=1;

		if (getenv("TIME_CRACK_MAXEPOCH"))
			TIME_CRACK_MAXEPOCH=atol(getenv("TIME_CRACK_MAXEPOCH"));
		else TIME_CRACK_MAXEPOCH=1;

		if (getenv("TIME_CRACK_MAXRUNS"))
			TIME_CRACK_MAXRUNS=atol(getenv("TIME_CRACK_MAXRUNS"));
		else TIME_CRACK_MAXRUNS=0;

		if (getenv("TIME_CRACK_MINEPOCH"))
			TIME_CRACK_MINEPOCH=atol(getenv("TIME_CRACK_MINEPOCH"));
		else TIME_CRACK_MINEPOCH=0;

		DIFF=TIME_CRACK_MAXEPOCH-TIME_CRACK_MINEPOCH;
		__gettimeofday(tv,tz);	
		START_TIME=tv->tv_sec;
	};
		

	b=__gettimeofday(tv,tz);

	if ( (TIME_CRACK_MAXRUNS==0) || (NUM_CALLS++ < TIME_CRACK_MAXRUNS) )
	{
		d=tv->tv_sec-TIME_CRACK_MINEPOCH;
		e=d%DIFF;
		tv->tv_sec=TIME_CRACK_MINEPOCH+e;
	} 


	if (TIME_CRACK_DEBUG)
	{
		fprintf(stderr,"Time Crack: GetTimeofDay()\n");
		fprintf(stderr,"Max: %d    Min: %d\n", TIME_CRACK_MAXEPOCH, TIME_CRACK_MINEPOCH);
		fprintf(stderr,"Returned: %d\n", tv->tv_sec);
		fprintf(stderr,"Num_Calls: %d\n", NUM_CALLS);
	}

	return b;
}

#ifdef XIMEOFDAY
time_t xime(time_t *t)
#else
time_t time(time_t *t)
#endif
{
	time_t h;

	struct timeval {
		long tv_sec;        /* seconds */
		long tv_usec;  /* microseconds */
	} tv;

	struct timezone {
		int  tz_minuteswest; /* minutes W of Greenwich */
		int  tz_dsttime;     /* type of dst correction */
	} tz;

#ifdef XIMEOFDAY
	getximeofday(&tv,&tz);
#else
	gettimeofday(&tv,&tz);
#endif

	h=tv.tv_sec;

	if (TIME_CRACK_DEBUG)
	{
		fprintf(stderr,"Time Crack: Time()\n");
		fprintf(stderr,"    max: %d    min: %d\n", TIME_CRACK_MAXEPOCH, TIME_CRACK_MINEPOCH);
	}

	if (t)
		(*t)=h;

	return h;
}

#ifdef SIGNAL_HACK
#ifdef XLARM
unsigned int xlarm(unsigned int seconds)
#else
unsigned int alarm(unsigned int seconds)
#endif
{ 
	if (getenv("TIME_CRACK_DEBUG"))
	{
		fprintf(stderr,"Time Crack: Alarm()\n");
		fprintf(stderr,"   seconds: %d\n", seconds);
		fprintf(stderr,"         b: %d\n", b);
	}
	return b;
}

#ifdef XLARM
int getiximer(int which, struct itimerval *value)
#else
int getitimer(int which, struct itimerval *value)
#endif
{
	if (getenv("TIME_CRACK_DEBUG"))
		fprintf(stderr,"Time Crack: GetiTimer()\n");
	return 0;
}

#ifdef XLARM
int setiximer(int which, const  struct  itimerval  *value, \
    struct itimerval *ovalue)
#else
int setitimer(int which, const  struct  itimerval  *value, \
    struct itimerval *ovalue)
#endif
{
	if (getenv("TIME_CRACK_DEBUG"))
	{
		fprintf(stderr,"Time Crack: SetITimer()\n");
		fprintf(stderr,"which: %d\n");
	}
}

#endif

